from safe.configs import Credentials
from pyathena import connect
from pyathena.pandas.util import as_pandas
from sqlalchemy import create_engine
import pandas as pd
from pyspark.sql import SparkSession
from pyspark import pandas as pds
import glob
import pathlib


class AthenaGo:
    def __init__(self):
        self._cred = Credentials().select('Athena', 'datalake')
        self._con = connect(s3_staging_dir=self._cred['s3_staging_dir'],
                            region_name=self._cred['region_name'])
        self._cursor = self._con.cursor()

    def read_sql(self, sql, verbose=False):
        if verbose:
            print(sql)
        return as_pandas(self._cursor.execute(sql))


class GoSpark:
    def __init__(self, host='local'):
        self.spark_session = SparkSession.builder.master(host).getOrCreate()
        self.pandas = pds


class MysqlGo:
    def __init__(self, base):
        self._buildurl(base)
        self._engine()
        self._pandas = pd

    def _buildurl(self, base):
        self._cred = Credentials().select('MySQL', base)
        self.url = f'mysql+pymysql://{self._cred["user"]}:{self._cred["password"]}@{self._cred["host"]}:{self._cred["port"]}/{self._cred["database"]}'

    def _engine(self):
        self._engine = create_engine(self.url)

    def spark(self):
        spark = GoSpark()
        self.spark_session = spark.spark_session
        self._pandas = spark.pandas
        return self

    def read_sql(self, sql):
        return self._pandas.read_sql(sql, con=self._engine)


class GoQuery:
    """This object load sql files and insert them into a dictionary to access via their key,
        which receives the same name as the file
    """
    def __init__(self, path: str):
        """
        :param path: Where your sql files are located
        """
        self.queries = {}
        self.path = path
        self._root = pathlib.Path().resolve().__str__()
        self._loadfiles()

    def _loadfiles(self):
        """
        Internal method that go through all files in directory and insert into a dictionary
        :return: None
        """
        for path in glob.glob(f"{self._root + '/' + self.path}/*.sql"):
            self._cachefile(path.split('/')[-1])

    def _cachefile(self, file: str):
        """
        Insert into dictionary
        :param file: File name
        :return: None
        """
        with open(self._root + '/' + self.path + '/' + file, 'r', encoding='utf-8') as line:
            self.queries[file.replace('.sql', '')] = line.read()



import pandas as pd
import shutil
import os
import pathlib


class GoStorage:
    def __init__(self, frame: pd.DataFrame, part=1):
        self._frame = frame
        self._part = part
        self._local = f'{pathlib.Path().resolve().__str__()}/temp'

    def _checkpart(self):
        return self._frame.shape[0]%self._part == 0

    def find_part(self, max, min):
        for i in range(max, min-1, -1):
            if self._frame.shape[0]%i == 0:
                self._size = self._frame.shape[0]/i
                self._part = i
                break
        return self

    def store(self):
        next = 0
        if os.path.exists(self._local):
            self.clear()
            os.makedirs(self._local)
        else:
            os.makedirs(self._local)

        for i in range(self._part):
            self._frame.iloc[:, next:next+next].to_csv(f'{self._local}/{i+1}_tmp.csv', index=False)
            next += self._size
        return self

    def clear(self):
        shutil.rmtree(self._local)


from time import time


class GoTimer:
    def __init__(self, verbose=True):
        self._verbose = verbose

    def start(self):
        self.start = time()
        self._total = 0.0
        self.schedule = []
        return self

    def checkpoint(self, reset=True):
        if self._verbose:
            print(f'{time()-self.start:.1f} sec')
        self.schedule.append(time()-self.start)
        self._total += time()-self.start
        if reset:
            self.reset()

    def reset(self):
        self.start = time()

    def stop(self, schedule=False):
        if self._verbose:
            print(f'{time()-self.start:.1f} sec')
        print(f'total time: {self._total:.1f} sec')
        if schedule:
            print(f'schedule: {self.schedule}')
        self.start = None


def clear_memory(frame, columns=[]):
    if len(columns) == 0:
        columns = frame.columns
    frame.drop(columns, axis=1)
    for column in columns:
        del frame[column]
        del column


import setuptools
import subprocess
import codecs

gowork_version = (
    subprocess.run(["git", "describe", "--tags"], stdout=subprocess.PIPE)
        .stdout.decode("utf-8")
        .strip()
)

setuptools.setup(
    name="gowork",
    version=gowork_version,
    author="Patrick Amaral",
    author_email="patrick.dev.atom@gmail.com",
    description="Library to help track your credentials and database engines",
    long_description=codecs.open('README.md', encoding='utf-8').read(),
    url="https://github.com/Phomint/GoWork",
    packages=setuptools.find_packages(),
    include_package_data=True,
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.6",
    license='GPLv3',
    keywords=['databases', 'athena', 'mysql'],
    install_requires=[
        "numpy",
        "pandas",
        "pyspark",
        "pyathena",
        "pymysql",
        "glob2",
        "sqlalchemy"
    ],
)